var targetItem        = 'gms';
var retrievingItem    = null;
var logging = true;
var animationHandler = null;
var cacheDays = 7;
var panelRange = { left:10, top:0, right:274, bottom:224, width:0, height:0 };
var pythonCommand = null;
var chartBounds = { x:0, y:0, width:0, height:0 };
var waitTimer = { element:null, count:0, wait:50, steps:12, timer:null };

function debug( message ) {
    if ( !logging ) return;
    if ( window.Logger ) {
        Logger.log( '' + message );
    } else {
        var div = document.getElementById( 'debugDiv' );
        if ( div ) {
            div.innerText = message;
        }
    }
}

function setup() {
    animationHandler = new AnimationButtonListener();
    importDefaultValues();

    createButton( document.getElementById( "buttonGMS" ), "GMS", new ChartButtonListener( "gms" ) );
    createButton( document.getElementById( "buttonChart" ), "CHRT", new ChartButtonListener( "chart" ) );
    createButton( document.getElementById( "buttonRain" ), "RAIN", new ChartButtonListener( "rain" ) );
    createButton( document.getElementById( "buttonTemp" ), "TEMP", new ChartButtonListener( "temp" ) );
    createButton( document.getElementById( "buttonPlay" ), "PLAY", animationHandler );//new AnimationButtonListener() );

    createInformationButton( document.getElementById( "front" ), -45, -20, flipToBehind, false );//function() { debug('to back' ); }, false );
    // done button
    createGenericButton( document.getElementById( 'done' ), tr( "Done" ), flipToFront );

    refreshChartBounds( 0, 0 );

    // prepare resizing
    var handle = document.getElementById( "resizeHandle" );
    if ( handle ) {
        handle.addEventListener( "mousedown", startResizing, true );
        handle.target = document.getElementById( "front" );
        handle.minimumSize = { width:250, height:180 };//235 };
        handle.maximumSize = { width:600, height:500 };
        var style = document.defaultView.getComputedStyle( handle.target );
        var width = parseInt( style.getPropertyValue( "width" ) );
        var height = parseInt( style.getPropertyValue( "height" ) );
        panelRange.right = width;
        panelRange.bottom = height;
        handle.currentSize = { width:width, height:height };
        handle.lastPosition = { x:0, y:0 };
    }

    var panel = document.getElementById( "messagePanel" );
    if ( panel ) {
        var psty = document.defaultView.getComputedStyle( panel );
        panelRange.width = parseInt( psty.getPropertyValue( "width" ) ) + 30;
        panelRange.height = parseInt( psty.getPropertyValue( "height" ) ) + 70;
    }

    // adjust panel position
    adjustMessagePanel();

    // localize strings
    _localizeMessages();

    // cache images
    for ( var i = 0; i < 12; i++ ) {
        (new Image()).src = "Images/Wait" + i + ".png";
    }

    setTimeout( loadInitialImage, 0 );
    invokeBackgroundLoaders();
}

function tr( key ) {
    try {
        var value = localizedStrings[ key ];
        if ( value ) {
            //debug( key + " -> " + value );
            return value;
        }
    } catch ( e ) {
    }
    return key;
}

function _localizeMessages() {
    var elements = document.getElementsByTagName( "span" );
    for ( var i = 0; i < elements.length; i++ ) {
        if ( elements[ i ].localizationKey ) {
            debug( i + " " + elements[ i ].localizationKey );
            debug( ">> " + tr( elements[ i ].localizationKey ) );
            elements[ i ].innerText = tr( elements[ i ].localizationKey );
        }
    }
    elements = document.getElementsByTagName( "OPTION" );
    for ( var i = 0; i < elements.length; i++ ) {
        if ( elements[ i ].localizationKey ) {
            debug( i + " " + elements[ i ].localizationKey );
            debug( ">> " + tr( elements[ i ].localizationKey ) );
            elements[ i ].innerText = tr( elements[ i ].localizationKey );
        }
    }
}

function showMessage( message ) {
    var messageField = document.getElementById( "messageText" );
    if ( messageField ) {
        messageField.innerHTML = message;
    } else {
        debug( message );
    }
}

function _getBounds( element ) {
    var st = document.defaultView.getComputedStyle( element );
    var left   = parseInt( st.getPropertyValue( "left" ) );
    var top    = parseInt( st.getPropertyValue( "top" ) );
    var width  = parseInt( st.getPropertyValue( "width" ) );
    var height = parseInt( st.getPropertyValue( "height" ) );
    return { x:left, y:top, width:width, height:height };
}

function importDefaultValues() {
    var px = parseInt( widget.preferenceForKey( "panelX" ) );
    var py = parseInt( widget.preferenceForKey( "panelY" ) );
    var panel = document.getElementById( "messagePanel" );
    if ( panel != null && px != null && py != null ) {
        panel.style.left = px + "px";
        panel.style.top = py + "py";
        panelRange.x = px;
        panelRange.y = py;
    }

    if ( widget.preferenceForKey( 'AnimationInterval' ) ) {
        //debug( widget.preferenceForKey( 'AnimationInterval' ) );
        var interval = parseInt( widget.preferenceForKey( 'AnimationInterval' ) );
        if ( interval != null ) {
            //animationInterval = interval;
            animationHandler.interval = interval;
            var select = document.getElementById( "animationSpeed" );
            var options = select.getElementsByTagName( "option" );
            for ( var i = 0; i < options.length; i++ ) {
                if ( parseInt( options[ i ].value ) == animationHandler.interval ) {//animationInterval ) {
                    options[ i ].setAttribute( "selected", "true" );
                } else {
                    options[ i ].removeAttribute( "selected" );
                }
            }
        }
    }

    if ( widget.preferenceForKey( "transparency" ) ) {
        var transparency = widget.preferenceForKey( "transparency" );
        var mainScreen = document.getElementById( "mainScreen" );
        mainScreen.style.opacity = transparency;
        var select = document.getElementById( "transparency" );
        var options = select.getElementsByTagName( "option" );
        for ( var i = 0; i < options.length; i++ ) {
            if ( parseInt( options.value ) == transparency ) {
                options[ i ].setAttribute( "selected", "true" );
            } else {
                options[ i ].removeAttribute( "selected" );
            }
        }
    }

    debug( "RETAIN : " + widget.preferenceForKey( "cacheretainperiod" ) );
    if ( widget.preferenceForKey( "cacheretainperiod" ) ) {
        var retainperiod = parseInt( widget.preferenceForKey( "cacheretainperiod" ) );
        debug( "retaining period : " + retainperiod );
        if ( isNaN( retainperiod ) == false ) {
            cacheDays = retainperiod;
        } else {
            cacheDays = "forever";
        }
        var select = document.getElementById( "imagecache" );
        if ( select ) {
            var children = select.childNodes;
            for ( var i = 0; i < children.length; i++ ) {
                if ( children.item( i ).tagName == "OPTION" ) {
                    if ( parseInt( children.item( i ).value ) == cacheDays ) {
                        children.item( i ).setAttribute( "selected", "true" );
                    } else {
                        children.item( i ).removeAttribute( "selected" );
                    }
                }
            }
        }
    }
}

// timer
function startWaitTimer() {
    if ( waitTimer.timer ) return;
    var wait = document.getElementById( "wait" );
    if ( wait ) {
        waitTimer.timer = setInterval( _tickWaitTimer, waitTimer.wait );
        waitTimer.element = wait;
        _tickWaitTimer();
        wait.style.display = "block";
    }
}
function _tickWaitTimer() {
    waitTimer.element.src = "Images/Wait" + waitTimer.count + ".png";
    waitTimer.count = ( waitTimer.count + 1 ) % waitTimer.steps;
}
function stopWaitTimer() {
    if ( waitTimer.timer == null ) return;
    waitTimer.element.style.display = "none";
    clearInterval( waitTimer.timer );
    waitTimer.timer = null;
}

function AnimationButtonListener() {
    this.playing  = false;
    this.timer    = null;
    this.interval = 100;
    this.filenames = null;
    this.counter   = 0;
    this.item = null;

    debug( "finding button" );
    try {
        var div = document.getElementById( "buttonPlay" );
        //debug( "div : " + div );
        //debug( "children : " + div.childNodes + " " + div.childNodes.length );
        this.button = div;//.getElementsByTagName( "div" )[ 1 ];//childNodes.item( 1 );
        //debug( this.button );
    } catch ( e ) {
        debug( "no button element :: " + e );
        this.button = null;
    }
    debug( "button is " + this.button );

    this.actionPerformed = function() {
        if ( this.playing ) {
            this.stopAnimation();
        } else {
            this.startAnimation();
        }
    };
    this.startAnimation = function() {
        //debug( "start animation" );
        startWaitTimer();
        if ( this.button ) {
            this.button.childNodes.item( 1 ).innerText = "STOP";
        }
        this.playing = true;
        this.item = targetItem;
        var command = "/usr/bin/python getchart.py " + this.item + " -list -retain";
        var pycom = widget.system( command, function( command ) { animationHandler._startAnimation( command ) } );
    }
    this._startAnimation = function( pycom ) {
        if ( this.playing == false ) return;
        var output = pycom.outputString;
        var error = pycom.errorString;
        var URIs = resolveURIs( output );
        this.filenames = URIs;
        this.counter = 0;
        //debug( output );
        //debug( URIs );
        stopWaitTimer();
        if ( this.filenames.length > 0 ) {
            this.timer = setInterval( _tickAnimation, this.interval );
            _tickAnimation();
        }
    };
    this.tick = function() {
//         debug( this.item );
//         debug( this.counter + " / " + this.filenames.length );
//         debug( this.filenames[ this.counter ] );
        if ( this.filenames[ this.counter ] ) {
            showGraphAtURI( this.item, this.filenames[ this.counter ] );
        }
        this.counter ++;
        if ( this.counter >= this.filenames.length ) {
            this.stopAnimation();
        }
    };
    this.stopAnimation = function() {
//        debug( "stop animation" );
        if ( this.button ) {
            this.button.childNodes.item( 1 ).innerText = "PLAY";
        }
        //if ( this.button ) this.button.innerText = "PLAY";
        this.playing = false;
        clearInterval( this.timer );
        this.timer = null;
    };
}
function _tickAnimation() {
    animationHandler.tick();
}


function ChartButtonListener( item ) {
    this.item = item;
    this.actionPerformed = function() {
        //debug( "listen " + this.item );
        targetItem = this.item;
        loadGraph( this.item );
    };
}


function loadInitialImage() {
    var availableImages = "";
    var listCommand = widget.system( "/usr/bin/python getchart.py gms -list -retain", null );
    var output = listCommand.outputString;
    var error  = listCommand.errorString;
    //debug( "error " + error + "\noutput " + output );
    if ( error != null && error.length > 0 ) {
        //showErrorMessage( error );
        debug( "Error:\n" + error );
    }
    if ( output != null && output.length > 0 ) {
        var lines = output.split( '\n' );
        if ( lines.length > 1 ) {
            var base_uri = lines[ 0 ];
            var uri = base_uri + lines[ 1 ] + '.jpg';
            showGraphAtURI( targetItem, uri );
        }
    } else {
//         debug( 'no images' );
//         document.getElementById( 'messageText' ).innerHTML = "ERROR";
//         debug( "ERROR" );
        showMessage( "no cached images" );
    }
    listCommand = null;
    debug( "load graph " + targetItem );
    loadGraph( targetItem );
    //document.getElementById( 'playButton' ).src = "Images/ButtonPlay1.png";
}

// load graph if no external script are running
function loadGraph( item ) {
    if ( pythonCommand != null ) {
        debug( 'already retrieving ' + retrievingItem + ", then loading '" + item + "' must wait for completion" );
        return;
    }
    if ( animationHandler ) {
        animationHandler.stopAnimation();
    }
    //stopAnimation();
    debug( item );
    var command = "/usr/bin/python getchart.py " + item;
    if ( item == targetItem ) {
        startWaitTimer();
    }
    if ( cacheDays == "forever" ) {
        command += " -retain";
    } else if ( cacheDays >= 0 ) {
        command += " -cacheperiod " + cacheDays;
    }
    debug( 'Python : ' + command );
    retrievingItem = item.substring( 0, item.length );
    debug( "retrieving " + retrievingItem );
    pythonCommand = widget.system( command, newGraphHandler );
}

function newGraphHandler() {
    stopWaitTimer();
    if ( pythonCommand == null ) {
        debug( 'no command' );
        return;
    }

    // display error 
    var error  = pythonCommand.errorString;
    if ( error != null && error.length > 0 ) {
        debug( error );
        showErrorMessage( error );
    }

    // set new image
    var output = pythonCommand.outputString;
    if ( output ) {
        debug( 'new graph : ' + output );
        var outputs = output.split( "\n" );
        var uri = outputs[ 0 ];
        if ( retrievingItem == targetItem ) {
            showGraphAtURI( targetItem, uri );
        } else {
            debug( "store '" + uri + "' to backyard : " + targetItem + '/' + retrievingItem );
        }
    } else {
        showMessage( "" );
    }

    pythonCommand = null;
    retrievingItem = null;
}


function showGraphAtURI( item, uri ) {
    var tempURI = uri + "?" + (new Date()).getTime();
    var itemName = resolveItemName( item );
    var date = getDateFromURI( uri );
    var graph = document.getElementById( "weatherimage" );
    
    graph.src = tempURI;
    graph.style.opacity = '1.0';

    // display information about the image
    if ( uri.indexOf( "http:" ) >= 0 ) {
        var d = new Date();
        if ( uri.indexOf( "http:" ) >= 0 ) {
            showMessage( itemName + "<br/>" + ( d.getMonth() + 1 ) + "/" + d.getDate() + "/" + d.getHours() );
        } else {
            showMessage( itemName + "<br/>" + date[ 1 ] + "/" + date[ 2 ] + "/" + date[ 3 ] );
        }
    } else {
        var r = new RegExp( /(\d\d(\d\d))(\d\d)(\d\d)\.jpg$/ );
        var m = r.exec( uri );
        if ( m ) {
            showMessage( itemName + "<br/>" + m[ 2 ] + "/" + m[ 3 ] + "/" + m[ 4 ] );
        }
    }
}



function refreshChartBounds( dx, dy ) {
    var container = document.getElementById( "weathergraph" );
    var image = document.getElementById( "weatherimage" );
    chartBounds = _getBounds( container );

    var x = chartBounds.x;
    var y = chartBounds.y;
    var w = chartBounds.width + dx;
    var h = chartBounds.height + dy;
    var hi, yi, wi, xi;
    if ( w * 3 > h * 4 ) {
        hi = h;
        yi = 0;
        wi = hi * 4 / 3;
        xi = ( w - wi ) / 2;
    } else {
        wi = w;
        xi = 0;
        hi = w * 3 / 4;
        yi = ( h - hi ) / 2;
    }

    image.style.left   = parseInt( xi ) + "px";
    image.style.top    = parseInt( yi ) + "px";
    image.style.width  = parseInt( wi ) + "px";
    image.style.height = parseInt( hi ) + "px";

    container.style.left   = parseInt( x ) + "px";
    container.style.top    = parseInt( y ) + "px";
    container.style.width  = parseInt( w ) + "px";
    container.style.height = parseInt( h ) + "px";
}

function startResizing( event ) {
    var handle = document.getElementById( "resizeHandle" );
    if ( handle ) {
        window.addEventListener( "mouseup", endResizing, true );
        window.addEventListener( "mousemove", resizeWindow, true );
        handle.lastPosition = { x:event.x, y:event.y };
    }    
}

function resizeWindow( event ) {
    var handle = document.getElementById( "resizeHandle" );
    if ( handle ) {
        var x = event.x;
        var y = event.y;
        var dx = x - handle.lastPosition.x;
        var dy = y - handle.lastPosition.y;
        var width = handle.currentSize.width + dx;
        var height = handle.currentSize.height + dy;
        if ( width < handle.minimumSize.width ) {
            dx += handle.minimumSize.width - width;
            width = handle.minimumSize.width;
        }
        if ( width > handle.maximumSize.width ) {
            dx += handle.maximumSize.width - width;
            width = handle.maximumSize.width;
        }
        if ( height < handle.minimumSize.height ) {
            dy += handle.minimumSize.height - height;
            height = handle.minimumSize.height;
        }
        if ( height > handle.maximumSize.height ) {
            dy += handle.maximumSize.height - height;
            height = handle.maximumSize.height;
        }
        panelRange.right = width;
        panelRange.bottom = height;
        //debug( "new size " + width + "," + height );
        handle.currentSize.width = width;
        handle.currentSize.height = height;
        handle.target.style.width = width + "px";
        handle.target.style.height = height + "px";
        window.resizeBy( dx, dy );
        handle.lastPosition = { x:event.x, y:event.y };
        refreshChartBounds( dx, dy );
        adjustMessagePanel();
    }
}

function endResizing( event ) {
    window.removeEventListener( "mouseup", endResizing, true );
    window.removeEventListener( "mousemove", resizeWindow, true );
}



function cleanUp() {
    var style = document.defaultView.getComputedStyle( document.getElementById( 'messagePanel' ) );
    var px = parseInt( style.getPropertyValue( 'left' ) );
    var py = parseInt( style.getPropertyValue( 'top' ) );
    widget.setPreferenceForKey( px, 'panelX' );
    widget.setPreferenceForKey( py, 'panelY' );
}

function resolveURIs( output ) {
    var URIs = new Array();
    if ( output && output.length > 0 ) {
        var lines = output.split( '\n' );
        var base_uri = lines[ 0 ];
        for ( var i = output.length - 1; i >= 1; i-- ) {
            if ( lines[ i ] && lines[ i ].length > 0 ) {
                var uri = base_uri + lines[ i ] + '.jpg';
                //debug( uri );
                URIs.push( uri );
            }
        }
    }
    return URIs;
}

// transit to preference
function flipToBehind(event) {
    var front  = document.getElementById( "front" ); 
    var behind = document.getElementById( "behind" ); 
    behind.style.width = panelRange.right + "px";
    behind.style.height = panelRange.bottom + "px";
    widget.prepareForTransition("ToBack"); 
    behind.style.display  = "block"; 
    front.style.display = "none"; 
    
    if (window.widget) {
        setTimeout ('widget.performTransition();', 10 );  
    }
}

// transit to main face
function flipToFront(event){
    var front  = document.getElementById( "front" ); 
    var behind = document.getElementById( "behind" ); 
    widget.prepareForTransition("ToFront"); 

    // get animation interval
    var interval = parseInt( document.getElementById( "animationSpeed" ).value );
    animationHandler.interval = interval;
    animationInterval = interval;
    behind.style.display  = "none"; 
    front.style.display = "block"; 

    // transparency
    var transparency = parseFloat( document.getElementById( "transparency" ).value );
    var mainScreen = document.getElementById( "mainScreen" );
    if ( mainScreen ) {
        mainScreen.style.opacity = transparency;
        if ( window.widget ) {
            widget.setPreferenceForKey( transparency, "transparency" );
        }
    }

    // image cache
    cacheDays = parseInt( document.getElementById( "imagecache" ).value );
    if ( isNaN( cacheDays ) ) {
        cacheDays = "forever";
    }

    // 
    if (window.widget) {
        widget.setPreferenceForKey( cacheDays, "cacheretainperiod" );
        widget.setPreferenceForKey( animationInterval, 'AnimationInterval' );
        setTimeout ('widget.performTransition();', 10 );  
    }
    
}

// translation for the future 
// function tr( key ) {
//     return key;
// }

// background loading
function invokeBackgroundLoaders() {
    var int_reload = 60000 * 20; // retrieve new data after every 20min
    var wait_chart = 300000;     // interval of each ping

    setTimeout( function() { loadChartWithInterval( 'gms', int_reload ); }, wait_chart );
    setTimeout( function() { loadChartWithInterval( 'chart', int_reload * 4 ); }, wait_chart * 2 );
    setTimeout( function() { loadChartWithInterval( 'rain', int_reload ); }, wait_chart * 3 );
    setTimeout( function() { loadChartWithInterval( 'temp', int_reload ); }, wait_chart * 4 );

}


function loadChartWithInterval( item, duration ) {
    debug( "set interval " + item  + " " + duration + " in loadChartWithInterval" );
    setInterval( function(){ loadGraph( item ); }, duration );
}

function resolveItemName( item ) {
    if ( item == 'gms' ) {
        return "衛星写真";
    } else if ( item == 'chart' ) {
        return "天気図";
    } else if ( item == 'rain' ) {
        return "降水量";
    } else if ( item == 'temp' ) {
        return "気温";
    } else {
        return "??";
    }
}

function getDateFromURI( uri ) {
    var ps = uri.lastIndexOf( '/' );
    var pp = uri.lastIndexOf( '.' );
    var year = "1900";
    var month = "00";
    var day = "00";
    var hour = "00";
    if ( ps == -1 ) ps = 0;
    if ( pp == -1 ) pp = uri.length;
    if ( pp - ps >= 11 ) {
        year = uri.substring( ps + 1, ps + 5 );
        month = uri.substring( ps + 5, ps + 7 );
        day = uri.substring( ps + 7, ps + 9 );
        hour = uri.substring( ps + 9, ps + 11 );
    }
    return [ year, month, day, hour ];
}

function showErrorMessage( message ) {
    if ( message == null || message == undefined ) {
        message = "エラー発生";
    }
    //document.getElementById( 'graph' ).src = "Images/Empty.png";
    document.getElementById( 'messageText' ).innerText = message;
}

var lastPos = { x:0, y:0 };
var panelPosition = { x:0, y:0 };
var panelDraggingFlag = false;

function adjustMessagePanel() {
    var panel = document.getElementById( 'messagePanel' );
    var style = document.defaultView.getComputedStyle( panel );
    var px = parseInt( style.getPropertyValue( 'left' ) );
    var py = parseInt( style.getPropertyValue( 'top' ) );
    if ( px > panelRange.right - panelRange.width ) {
        px = panelRange.right - panelRange.width;
        panel.style.left = px + "px";
    }
    if ( py > panelRange.bottom - panelRange.height ) {
        py = panelRange.bottom - panelRange.height;
        panel.style.top = py + "px";
    }
}

function panelDragStart(event) {
    var style = document.defaultView.getComputedStyle( document.getElementById( 'messagePanel' ) );
    var px = parseInt( style.getPropertyValue( 'left' ) );
    var py = parseInt( style.getPropertyValue( 'top' ) );

    lastPos = { x:event.x, y:event.y };
    panelPosition = { x:px, y:py };

    window.addEventListener( 'mousemove', panelDragging, true );
    window.addEventListener( 'mouseup', panelDragEnd, true );
}

function panelDragging( evnet ) {
    var dx = event.x - lastPos.x;
    var dy = event.y - lastPos.y;
    var elem = document.getElementById( "messagePanel" );
    panelPosition.x += dx;
    panelPosition.y += dy;
    if ( panelPosition.x < panelRange.left ) {
        panelPosition.x = panelRange.left;
    }
    if ( panelPosition.x > panelRange.right - panelRange.width ) {
        panelPosition.x = panelRange.right - panelRange.width;
    }
    if ( panelPosition.y < panelRange.top ) {
        panelPosition.y = panelRange.top;
    }
    if ( panelPosition.y > panelRange.bottom - panelRange.height ) {
        panelPosition.y = panelRange.bottom - panelRange.height;
    }
    elem.style.left = panelPosition.x + "px";
    elem.style.top  = panelPosition.y + "px";
    lastPos.x = event.x;
    lastPos.y = event.y;
}

function panelDragEnd(event) {
    window.removeEventListener( 'mousemove', panelDragging, true );
    window.removeEventListener( 'mouseup', panelDragEnd, true );
}
